/*=============================================================================
	Copyright (c) 1999-2004 pmG Worldwide, LLC.

  Author: Lyle Milton
  Date: 06/25/04


  LightData Utility

	NOTE: This module simply retrieves light data from shader_globals and
	makes it available to shaders.  It's most useful with using Light Materials.

=============================================================================*/
#include "globals.h"

//Defines,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
#define MLD_CLAMP	(1<<0) //clamp the distance to 0.0 - 1.0

#define MLD_MINDIST		0.000001 //min range for max_distnace
#define MLD_MAXDIST		100000.0 //max range for max_distnace
//`````````````````````````````````````````````````````````````````````````````


//Local data struct,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
typedef struct My_LightData_St
{
	int				flags;
	FXobject		lightID;
	double			max_distance;

} My_LightData;
//`````````````````````````````````````````````````````````````````````````````

//Shader Callbacks,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
static FXvoid ShaderDataInit(My_LightData *mld);

static FX_CONCALLBACK(TargetLight_ObjectPopupList);
static FX_CONCALLBACK(MaxDistance_DragFloat);
static FX_CONCALLBACK(ClampDistance_Boolean);

static FX_SHADERINITINPUTS(ShaderInitInputs);
static FX_SHADEREVALUATE(ShaderEvaluate);
//`````````````````````````````````````````````````````````````````````````````



//=======================================================================================
//LightAccess
//		shader access callback
//=======================================================================================
FX_ACCESSFUNC(LightAccess)
{
	FXshader		shader = (FXshader)ID;
	My_LightData	*mld = NULL;

	if (level==ACCESS_OBJECT)
	{
		switch(entry)
		{
			//Perform shader object creation,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
			case O_CREATE:
				{
					//#1. Create & Init shader data------------------------------------------------
					if (!(mld = (My_LightData*)fxDataCreate(shader, &lightdataID, 0, sizeof(My_LightData))))
					{
						//Send error message,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
						fxMessageSend(lightdataID, "GlobalLight", FX_MESSAGE_POST|FX_MESSAGE_ERROR,
							"Couldn't allocate necessary object data!", NULL);

						return CONTINUE_ENTRY;
					}

					ShaderDataInit(mld);
					fxObjectSetTypeData(shader, mld, 0);
					//`````````````````````````````````````````````````````````````````````````````

					//Create Outputs-----------------------------------------------------------
					//L_OUT_POS			= fxShaderCreateOutput(shader, "Position", FX_IOTYPE_POINT, 0);
					L_OUT_DIR			= fxShaderCreateOutput(shader, "Ray_Direction", FX_IOTYPE_VECTOR, 0);
					L_OUT_DST			= fxShaderCreateOutput(shader, "Ray_Length", FX_IOTYPE_FLOAT, 0);
					//L_OUT_INC			= fxShaderCreateOutput(shader, "Incidence", FX_IOTYPE_FLOAT, 0);
					//`````````````````````````````````````````````````````````````````````````````

					//Set Shader Callbacks-----------------------------------------------------
					fxShaderCallback(shader, FX_SHADERFUNC_INITINPUTS, &ShaderInitInputs);
					fxShaderCallback(shader, FX_SHADERFUNC_EVALUATE, &ShaderEvaluate);
					//`````````````````````````````````````````````````````````````````````````
				}

				break;

			case O_DESTROY:
				fxObjectSetTypeData(shader, NULL, 0);
				break;

			case O_SAVE:
				{
					if (!(mld = fxObjectGetTypeData(shader, 0))) return CONTINUE_ENTRY;

					fxFormatNew(ai, "LightData", 0);

						fxArgSaveInt(ai,	"Flags",	mld->flags);
						fxArgSaveDouble(ai, "MaxDist",	mld->max_distance);

						if (mld->lightID)
							fxArgSaveObj(ai, "Target",	mld->lightID);
				}
				break;

			case O_LOAD:
				{
					if (!(mld = fxObjectGetTypeData(shader, 0))) return CONTINUE_ENTRY;

					ShaderDataInit(mld); //always initialize the data before loading

						fxArgLoadInt(ai,	"Flags",	&mld->flags);
						fxArgLoadDouble(ai, "MaxDist",	&mld->max_distance);
						fxArgLoadObj(ai,	"Target",	&mld->lightID);

						mld->max_distance = FX_CLAMP(mld->max_distance, MLD_MINDIST, MLD_MAXDIST); //just a precaution;})
				}
				break;

			default:
				break;
		}
	}
	else if (level==ACCESS_INTERFACE)
	{
		switch(entry)
		{
			case IN_CREATE:
				{
					FXcontrol		block, obj_popup, drag_float, bool_int;

					static FXchar	*filter_sl[] = {FX_TOOLTYPE_LIGHT, NULL};

					//LightData ----------------------------------------------------------------
					block = fxCC_Block("LightData", NULL, ACC_VALUE, 0);


						obj_popup = fxCC_ObjectPopup("Target Light", &TargetLight_ObjectPopupList, ACC_UPDATE,
							
								176, filter_sl, NULL, NULL, OBJPOPUP_ALL|OBJPOPUP_FILTER_TYPESHOW);

						drag_float = fxCC_DragFloat("Max Distance", &MaxDistance_DragFloat, ACC_UPDATE,
							
								MLD_MINDIST, MLD_MAXDIST, 0.010000, 0);
						
						bool_int = fxCC_Bool("Clamp Distance", &ClampDistance_Boolean, ACC_UPDATE);

						fxConSetPos(obj_popup, 76, 4);
						fxConSetPos(drag_float, 76, 25);
						fxConSetPos(bool_int, 144, 25);


					//Assign the blocks,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
					fxInterfaceBlockSet(FX_MODE_RENDER, FX_MODE_RENDER, FX_MBLOCK_SHADER, FX_NULLID);
					fxInterfaceBlockSet(FX_MODE_RENDER, NULL, NULL, block);
					fxInterfaceBlockSet(FX_MODE_RENDER, FX_MODE_RENDER, FX_MBLOCK_SPLINE, FX_NULLID);
				}
				break;

			default:
				break;
		}
	}

	return ENTRY_OK;
}


//=======================================================================================
//ShaderDataInit
//		Initialize the shader data.
//=======================================================================================
static FXvoid ShaderDataInit(My_LightData *mld)
{
	mld->flags = MLD_CLAMP;
	mld->lightID = FX_NULLID;
	mld->max_distance = 1.0;
}





//===================================================================================== TargetLight_ObjectPopupList
static FX_CONCALLBACK(TargetLight_ObjectPopupList)
{
	FXshader		shader = (FXshader)ctl_data;
	My_LightData	*mld = fxShaderGetTypeData(shader, 0);

	if (!mld) return CONTINUE_ENTRY;

	switch(entry)
	{
		case ACC_VALUE:
			fxConGetObj(ctl, &mld->lightID);
			break;

		case ACC_UPDATE:
			fxConSetObj(ctl, mld->lightID);
			break;
	}

	return CONTINUE_ENTRY;
}


//===================================================================================== MaxDistance_DragFloat
static FX_CONCALLBACK(MaxDistance_DragFloat)
{
	FXshader		shader = (FXshader)ctl_data;
	My_LightData	*mld = fxShaderGetTypeData(shader, 0);

	if (!mld) return CONTINUE_ENTRY;

	switch(entry)
	{
		case ACC_VALUE:
			fxConGetDouble(ctl, &mld->max_distance);
			mld->max_distance = FX_CLAMP(mld->max_distance, MLD_MINDIST, MLD_MAXDIST); //just a precaution;})
			break;

		case ACC_UPDATE:
			fxConSetDouble(ctl, mld->max_distance);
			break;
	}

	return CONTINUE_ENTRY;
}


//===================================================================================== ClampDistance_Boolean
static FX_CONCALLBACK(ClampDistance_Boolean)
{
	FXshader		shader = (FXshader)ctl_data;
	My_LightData	*mld = fxShaderGetTypeData(shader, 0);
	int				i=0;

	if (!mld) return CONTINUE_ENTRY;

	switch(entry)
	{
		case ACC_VALUE:
			fxConGetInt(ctl, &i);

			if (i)
				mld->flags |= MLD_CLAMP;
			else
				mld->flags &= (~MLD_CLAMP);

			break;

		case ACC_UPDATE:
			fxConSetInt(ctl, ((mld->flags&MLD_CLAMP)?1:0));
			break;
	}

	return CONTINUE_ENTRY;
}


//=======================================================================================
//ShaderInitInputs
//		Initialize the shader node inputs.
//=======================================================================================
static FX_SHADERINITINPUTS(ShaderInitInputs)
{
	return;
}

//=======================================================================================
//ShaderEvaluate
//		Evaluate the shader and set the outputs.
//=======================================================================================
static FX_SHADEREVALUATE(ShaderEvaluate)
{
	My_LightData	*mld = fxShaderGetTypeData(shader, 0);

	if (FX_OUTPUT_REQUEST(L_OUT_DIR))
		l_out_dir = _Ld;

	if (FX_OUTPUT_REQUEST(L_OUT_DST))
	{
		l_out_dst = _Ll/(FXfloat)mld->max_distance;

		if (mld->flags&MLD_CLAMP)
		{
			l_out_dst = FX_CLAMP(l_out_dst, 0.0f, 1.0f);
		}
	}

	return;
}

