/*!=============================================================================
	Copyright (c) 2003 pmG WorldWide, LLC.

	\file Point.c
	\author Ron Griswold
	
	Created: 9/11/03

	CVS: $Revision$	$Date$

	Description: Point Utility Shader

=============================================================================*/
#include "Plugin.h"

//////////////////////////////////////////////////////
// Globals
//////////////////////////////////////////////////////

// Data Keys
static int		DATA_KEYID		= 0;					// fxData* keyID
static int *	DATA_KEY		= &DATA_KEYID;			// fxData* key

// Shader Input IDs

// Shader Output IDs
static FXint	OUT_P			= 0;
static FXint	OUT_N			= 0;
static FXint	OUT_Ng			= 0;
static FXint	OUT_U			= 0;
static FXint	OUT_V			= 0;
static FXint	OUT_Ld			= 0;
static FXint	OUT_Li			= 0;
static FXint	OUT_Cs			= 0;
static FXint	OUT_Os			= 0;

//////////////////////////////////////////////////////
// State Data
//////////////////////////////////////////////////////
// (none)

//////////////////////////////////////////////////////
// Shader Callbacks
//////////////////////////////////////////////////////
static FX_SHADERINITINPUTS(ShaderInitInputs)
// FXvoidf f(FXshader shader, FX_ShaderGlobals *shader_globals, FX_ShaderParam *input)
{
	// no inputs
}

static FX_SHADEREVALUATE(ShaderEvaluate)
// FXvoidf f(FXshader shader, FX_ShaderGlobals *shader_globals, FX_ShaderParam *input, FX_ShaderParam *output)
{
	output[OUT_P].vec.x = _P.x;
	output[OUT_P].vec.y = _P.y;
	output[OUT_P].vec.z = _P.z;

	output[OUT_N].vec.x = _N.x;
	output[OUT_N].vec.y = _N.y;
	output[OUT_N].vec.z = _N.z;

	output[OUT_Ng].vec.x = _Ng.x;
	output[OUT_Ng].vec.y = _Ng.y;
	output[OUT_Ng].vec.z = _Ng.z;

	output[OUT_Ld].vec.x = _Ld.x;
	output[OUT_Ld].vec.y = _Ld.y;
	output[OUT_Ld].vec.z = _Ld.z;
	
	output[OUT_U].f = _u;
	output[OUT_V].f = _v;

	output[OUT_Li].color.r = _Li.r;
	output[OUT_Li].color.g = _Li.g;
	output[OUT_Li].color.b = _Li.b;

	output[OUT_Cs].color.r = _Cs.r;
	output[OUT_Cs].color.g = _Cs.g;
	output[OUT_Cs].color.b = _Cs.b;

	output[OUT_Os].color.r = _Os.r;
	output[OUT_Os].color.g = _Os.g;
	output[OUT_Os].color.b = _Os.b;

}

//////////////////////////////////////////////////////
// Control Callbacks
//////////////////////////////////////////////////////

//////////////////////////////////////////////////////
// Access Function
//////////////////////////////////////////////////////
FX_ACCESSFUNC( Point_Access )
// FXintf f(FX_AccessInfo *ai, FXentity ID, FXint level, FXint64 entry)
{
	FXshader	shader = (FXshader)ID;

	switch( level )
	{
	case ACCESS_OBJECT:
		{
			switch( entry )
			{
			case O_CREATE:
				{
					// Create State Data
					// (none)

					// Initialize State Data
					// (none)

					// Set State Data (Necessary for proper shader evaluation)
					// (none)

					// Setup Channels
					// (none)

					// Link Channels to State Data
					// (none)

					// Create Inputs
					// (none)

					// Create Outputs
					OUT_P		= fxShaderCreateOutput( shader, "Point",			FX_IOTYPE_POINT, FX_NOFLAG );
					OUT_N		= fxShaderCreateOutput( shader, "SmootNormal",		FX_IOTYPE_VECTOR, FX_NOFLAG );
					OUT_Ng		= fxShaderCreateOutput( shader, "GeomNormal",		FX_IOTYPE_VECTOR, FX_NOFLAG );
					OUT_U		= fxShaderCreateOutput( shader, "u",				FX_IOTYPE_FLOAT, FX_NOFLAG );
					OUT_V		= fxShaderCreateOutput( shader, "v",				FX_IOTYPE_FLOAT, FX_NOFLAG );
					OUT_Ld		= fxShaderCreateOutput( shader, "LightDirection",	FX_IOTYPE_VECTOR, FX_NOFLAG );
					OUT_Li		= fxShaderCreateOutput( shader, "LightIntensity",	FX_IOTYPE_COLOR, FX_NOFLAG );
					OUT_Cs		= fxShaderCreateOutput( shader, "SurfaceColor",		FX_IOTYPE_COLOR, FX_NOFLAG );
					OUT_Os		= fxShaderCreateOutput( shader, "SurfaceOpacity",	FX_IOTYPE_COLOR, FX_NOFLAG );

					// Set Shader Callbacks
					fxShaderCallback( shader, FX_SHADERFUNC_INITINPUTS, &ShaderInitInputs );
					fxShaderCallback( shader, FX_SHADERFUNC_EVALUATE, &ShaderEvaluate );

				} // O_CREATE
				break;

			case O_DESTROY:
				{

				} // O_DESTROY
				break;
			}
		} // ACCESS_OBJECT
		break;

	case ACCESS_INTERFACE:
		{
			switch( entry )
			{
			case IN_CREATE:
				{
					//Assign the Blocks
					fxInterfaceBlockSet(FX_MODE_RENDER, FX_MODE_RENDER, FX_MBLOCK_SHADER, FX_NULLID);

				} // O_CREATE
				break;
			}
		} // ACCESS_INTERFACE
		break;
	}

	return ENTRY_OK;
}