/*=============================================================================
	Copyright (c) 1999-2003 pmG Worldwide, LLC.

	File:	messiah_motion.h
	Author:	Lyle Milton
	Date:	01/01/03

		last modified: 05/09/03 - LM
			- fxKey* calculation functions
				+ these functions are based on some of the expression
				  functions, e.g. KneeConstraint, IK2D, etc.
			- fxChannelsBlend
				+ allows you to blend a value into a channel using the
				  FX_CHANOP_* operators.  Think of it like changing the
				  operator type for a channel in Command mode

=============================================================================*/

#ifdef _MESSIAH_API_MOTION

#ifndef __MESSIAH_MOT
#define __MESSIAH_MOT


#ifndef __MESSIAH_HIDE_TYPES

	#define NUM_CHAN					12
	#define NUM_CHAN_EX					32

	//Channel Types,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_CHANTYPE_POSITION			0
	#define FX_CHANTYPE_ROTATION			1
	#define FX_CHANTYPE_SCALE				2
	#define FX_CHANTYPE_PIVOT				3
	#define FX_CHANTYPE_VALUE				4
	#define FX_CHANTYPE_COLOR				7 //0.0 -> 255.0


	#define FX_CHAN_0					0
	#define FX_CHAN_1					1
	#define FX_CHAN_2					2
	#define FX_CHAN_3					3
	#define FX_CHAN_4					4
	#define FX_CHAN_5					5
	#define FX_CHAN_6					6
	#define FX_CHAN_7					7
	#define FX_CHAN_8					8
	#define FX_CHAN_9					9

	#define FX_CHAN_10					10
	#define FX_CHAN_11					11
	#define FX_CHAN_12					12
	#define FX_CHAN_13					13
	#define FX_CHAN_14					14
	#define FX_CHAN_15					15
	#define FX_CHAN_16					16
	#define FX_CHAN_17					17
	#define FX_CHAN_18					18
	#define FX_CHAN_19					19

	#define FX_CHAN_20					20
	#define FX_CHAN_21					21
	#define FX_CHAN_22					22
	#define FX_CHAN_23					23
	#define FX_CHAN_24					24
	#define FX_CHAN_25					25
	#define FX_CHAN_26					26
	#define FX_CHAN_27					27
	#define FX_CHAN_28					28
	#define FX_CHAN_29					29


	#define FX_X_POSITION				0
	#define FX_Y_POSITION				1
	#define FX_Z_POSITION				2
	#define FX_HEADING					3
	#define FX_PITCH					4
	#define FX_BANK						5
	#define FX_X_SCALE					6
	#define FX_Y_SCALE					7
	#define FX_Z_SCALE					8
	#define FX_X_PIVOT					9
	#define FX_Y_PIVOT					10
	#define FX_Z_PIVOT					11


	#define FX_X_WPOSITION				32
	#define FX_Y_WPOSITION				33
	#define FX_Z_WPOSITION				34

	#define FX_POSITION					35
	#define FX_ROTATION					36
	#define FX_SCALE					37
	#define FX_PIVOT					38
	#define FX_MOTION_CHANNELS			39
	#define FX_OBJECT_CHANNELS			40
	#define FX_ALL_CHANNELS				41
	#define FX_EX_CHANNELS				42

	#define FX_WORLD_POSITION			43
	#define FX_WORLD_ROTATION			44
	#define FX_WORLD_SCALE				45
	#define FX_WORLD_PIVOT				46


	#define FX_MATRIX_LOCAL				49
	#define FX_MATRIX_WORLD				50
	#define FX_MATRIX_WORLD_FILTERED	51



	#define FX_X_POSITION_BIT			(1<<FX_X_POSITION)
	#define FX_Y_POSITION_BIT			(1<<FX_Y_POSITION)
	#define FX_Z_POSITION_BIT			(1<<FX_Z_POSITION)
	#define FX_HEADING_BIT				(1<<FX_HEADING)
	#define FX_PITCH_BIT				(1<<FX_PITCH)
	#define FX_BANK_BIT					(1<<FX_BANK)
	#define FX_X_SCALE_BIT				(1<<FX_X_SCALE)
	#define FX_Y_SCALE_BIT				(1<<FX_Y_SCALE)
	#define FX_Z_SCALE_BIT				(1<<FX_Z_SCALE)
	#define FX_X_PIVOT_BIT				(1<<FX_X_PIVOT)
	#define FX_Y_PIVOT_BIT				(1<<FX_Y_PIVOT)
	#define FX_Z_PIVOT_BIT				(1<<FX_Z_PIVOT)

	#define FX_POSITION_BITS			(FX_X_POSITION_BIT|FX_Y_POSITION_BIT|FX_Z_POSITION_BIT)
	#define FX_ROTATION_BITS			(FX_HEADING_BIT|FX_PITCH_BIT|FX_BANK_BIT)
	#define FX_SCALE_BITS				(FX_X_SCALE_BIT|FX_Y_SCALE_BIT|FX_Z_SCALE_BIT)
	#define FX_PIVOT_BITS				(FX_X_PIVOT_BIT|FX_Y_PIVOT_BIT|FX_Z_PIVOT_BIT)
	#define FX_MOTION_CHANNELS_BITS		(FX_POSITION_BITS|FX_ROTATION_BITS)
	#define FX_OBJECT_CHANNELS_BITS		(FX_POSITION_BITS|FX_ROTATION_BITS|FX_SCALE_BITS)
	#define FX_OBJECT_ALL_CHANNELS_BITS	(FX_POSITION_BITS|FX_ROTATION_BITS|FX_SCALE_BITS|FX_PIVOT_BITS)
	#define FX_ALL_CHANNELS_BITS		(0xffffffff)
	#define FX_EX_CHANNELS_BITS			(FX_ALL_CHANNELS_BITS^FX_OBJECT_CHANNELS_BITS)


	//Channel Operators,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_CHANOP_EQUAL				0
	#define FX_CHANOP_ADD				1
	#define FX_CHANOP_SUB				2
	#define FX_CHANOP_MULT				3
	#define FX_CHANOP_DIV				4
	#define FX_CHANOP_MAX				5
	#define FX_CHANOP_MIN				6


	//Motion End Behavior,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_END_RESET				0
	#define FX_END_STOP					1
	#define FX_END_REPEAT				2
	#define FX_END_REPEAT_LOCAL			3
	#define FX_END_PINGPONG				4
	#define FX_END_ACCUMULATE			5
	#define FX_END_ACCUMULATE_LOCAL		6
	#define FX_END_WRAP					7
	#define FX_END_PINGPONG_LOCAL		8


	//Motion Curve Type,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_CURVETYPE_TCB			0
	#define FX_CURVETYPE_BEZIER			1
	#define FX_CURVETYPE_LINEAR			2
	#define FX_CURVETYPE_STEP			3

	#define FX_TANGENTS_BREAK			(1<<0)
	#define FX_TANGENTS_LOCK_LENGTH		(1<<1)


	//flags for fxChannels*,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
	#define FX_CHANXFER_SETUP			(1<<31)

	//flags for fxChannelOptions
	#define FX_CHANFLAG_STATIC			(1<<0)
	#define FX_CHANFLAG_HIDE			(1<<1)
	#define FX_CHANFLAG_SHOWKEYS		(1<<2) //SHOW KEYS IN GRAPH WHEN MULTI-SELECTED

	//return values for fxKeyCreate
	#define FX_KEY_NEW					1
	#define FX_KEY_EXISTS				2

	//type "flags" for fxMatrixGet
	#define FX_M3X3						0
	#define FX_M4X4						1

	//alternative matrix axis names
	#define fxMatrixRotateHeading			fxMatrixRotateY
	#define fxMatrixRotatePitch				fxMatrixRotateX
	#define fxMatrixRotateBank				fxMatrixRotateZ

#endif //__MESSIAH_HIDE_TYPES


FX_API(FXint, fxMotionNumKeys)(FXobject, FXint *num_keys, FXint flags);

FX_API(FXint, fxMotionUpdate)(FXobject, FXint flags);

FX_API(FXint, fxChannelsGet)(FXobject, FXint channel_ID, FXdouble frame, FXdouble *channels, FXint flags);

FX_API(FXint, fxChannelsSet)(FXobject, FXint channel_ID, FXdouble *channels, FXint flags);

FX_API(FXint, fxChannelsBlend)(FXobject, FXint channelID, FXdouble *channels, FXint blend_op, FXint flags);

FX_API(FXint, fxChannelActiveGet)(FXobject, FXint *channel, FXint flags);

FX_API(FXint, fxChannelActiveSet)(FXobject, FXint channel, FXint flags);

FX_API(FXint, fxChannelNumKeys)(FXobject, FXint channel, FXint *num_keys, FXint flags);

FX_API(FXint, fxChannelSetup)(FXobject, FXchar *name, FXint channel, FXint channel_type, FXdouble def, FXdouble min, FXdouble max, FXint flags);

FX_API(FXint, fxChannelDataLink)(FXobject, FXint channel, FXint value_type, FXvoid *value, FXint flags);

FX_API(FXint, fxChannelName)(FXobject, FXint channel, FXchar *buffer, FXint buffer_length, FXint flags);

FX_API(FXint, fxChannelRename)(FXobject, FXint channel, FXchar *name, FXint flags);

FX_API(FXint, fxChannelOptions)(FXobject, FXint channel, FXint *flags);

FX_API(FXint, fxChannelOptionsSet)(FXobject, FXint channel, FXint flags);

FX_API(FXint, fxChannelOptionsClear)(FXobject, FXint channel, FXint flags);

FX_API(FXint, fxKeyEditBegin)(FXmodule, FXobject, FXint flags);

FX_API(FXint, fxKeyEditEnd)(FXmodule, FXobject, FXint flags);

FX_API(FXint, fxKeyChannelsGet)(FXobject, FXint keyID, FXint channel_ID, FXdouble *channels, FXint flags);

FX_API(FXint, fxKeyChannelsSet)(FXobject, FXint keyID, FXint channel_ID, FXdouble *channels, FXint flags);

FX_API(FXint, fxKeyChannelsGetBezierHandleIn)(FXobject, FXint keyID, FXint channel_ID, FXdouble *channels, FXint flags);
FX_API(FXint, fxKeyChannelsSetBezierHandleIn)(FXobject, FXint keyID, FXint channel_ID, FXdouble *channels, FXint flags);

FX_API(FXint, fxKeyChannelsGetBezierHandleOut)(FXobject, FXint keyID, FXint channel_ID, FXdouble *channels, FXint flags);
FX_API(FXint, fxKeyChannelsSetBezierHandleOut)(FXobject, FXint keyID, FXint channel_ID, FXdouble *channels, FXint flags);

FX_API(FXint, fxKeyFrameGetBezierHandleIn)(FXobject, FXint keyID, FXint channel, FXdouble *frame, FXint flags);
FX_API(FXint, fxKeyFrameSetBezierHandleIn)(FXobject, FXint keyID, FXint channel, FXdouble frame, FXint flags);

FX_API(FXint, fxKeyFrameGetBezierHandleOut)(FXobject, FXint keyID, FXint channel, FXdouble *frame, FXint flags);
FX_API(FXint, fxKeyFrameSetBezierHandleOut)(FXobject, FXint keyID, FXint channel, FXdouble frame, FXint flags);


FX_API(FXint, fxKeyInclusionGet)(FXobject, FXint keyID, FXint *channel_flags, FXint flags);

FX_API(FXint, fxKeyInclusionSet)(FXobject, FXint keyID, FXint channel_flags, FXint flags);

FX_API(FXint, fxKeySelected)(FXobject, FXint keyID, FXint channel, FXint flags);

FX_API(FXint, fxKeySelect)(FXobject, FXint keyID, FXint channel, FXint flags);

FX_API(FXint, fxKeyDeselect)(FXobject, FXint keyID, FXint channel, FXint flags);

FX_API(FXint, fxKeyActiveGet)(FXobject, FXint *keyID, FXint flags);

FX_API(FXint, fxKeyActiveSet)(FXobject, FXint keyID, FXint flags);

FX_API(FXint, fxKeyExists)(FXobject, FXdouble frame, FXint *keyID, FXint flags);

FX_API(FXint, fxKeyFrameGet)(FXobject, FXint keyID, FXdouble *frame, FXint flags);

FX_API(FXint, fxKeyTimeGet)(FXobject, FXint keyID, FXdouble *time_in_seconds, FXint flags);

FX_API(FXint, fxKeyCreate)(FXobject, FXdouble frame, FXint *keyID, FXint flags);

FX_API(FXint, fxKeyChannelCreate)(FXobject, FXdouble frame, FXint *keyID, FXint channel_ID, FXdouble *channels, FXint flags);

FX_API(FXint, fxKeyTCBGet)(FXobject objID, FXint keyID, FXint channel_ID, FXdouble *t, FXdouble *c, FXdouble *b, FXint flags);

FX_API(FXint, fxKeyTCBSet)(FXobject objID, FXint keyID, FXint channel_ID, FXdouble *t, FXdouble *c, FXdouble *b, FXint flags);

FX_API(FXint, fxKeyDelete)(FXobject, FXint keyID, FXint flags);

FX_API(FXint, fxChannelEndSet)(FXobject, FXint channel_ID, FXint end_type, FXint flags);

FX_API(FXint, fxChannelEndGet)(FXobject, FXint channel_ID, FXint *end_type, FXint flags);

FX_API(FXint, fxKeyCurveSet)(FXobject, FXint keyID, FXint channel_ID, FXint curve_type, FXint flags);

FX_API(FXint, fxKeyCurveGet)(FXobject, FXint keyID, FXint channel_ID, FXint *curve_type, FXint flags);

FX_API(FXint, fxKeyTangentsSet)(FXobject, FXint keyID, FXint channel_ID, FXint tangent_flags, FXint flags);

FX_API(FXint, fxKeyTangentsGet)(FXobject, FXint keyID, FXint channel_ID, FXint *tangent_flags, FXint flags);

FX_API(FXint, fxMatrixGet)(FXobject, FXint chan_mat, FXdouble frame, FXmatrix m, FXint flags);

FX_API(FXint, fxMatrixHPB)(FXmatrix, FXvecd hpb, FXint flags);

FX_API(FXint, fxMatrixInit)(FXmatrix, FXchannels, FXint flags);

FX_API(FXint, fxMatrixTranslate)(FXmatrix, FXvecd, FXint flags);

FX_API(FXint, fxMatrixScale)(FXmatrix, FXvecd, FXint flags);

FX_API(FXint, fxMatrixRotateX)(FXmatrix, FXdouble p, FXint flags);

FX_API(FXint, fxMatrixRotateY)(FXmatrix, FXdouble h, FXint flags);

FX_API(FXint, fxMatrixRotateZ)(FXmatrix, FXdouble b, FXint flags);

FX_API(FXint, fxKeyKneeConstraintPosV)(FXobject root, FXobject knee, FXvecd goal, FXint channel);

FX_API(FXint, fxKeyKneeConstraintPos)(FXobject root, FXobject knee, FXdouble x, FXdouble y, FXdouble z, FXint channel);

FX_API(FXint, fxKeyKneeConstraint)(FXobject root, FXobject knee, FXobject goal, FXint channel);


FX_API(FXint, fxKeyTargetPosV)(FXobject endeff, FXvecd  goal);

FX_API(FXint, fxKeyTargetPos)(FXobject endeff, FXdouble x, FXdouble y, FXdouble z);

FX_API(FXint, fxKeyTarget)(FXobject endeff, FXobject goal);


FX_API(FXint, fxKeyIK2DPosV)(FXobject root, FXobject endeff, FXvecd goal, FXint align);

FX_API(FXint, fxKeyIK2DPos)(FXobject root, FXobject endeff, FXdouble x, FXdouble y, FXdouble z, FXint align);

FX_API(FXint, fxKeyIK2D)(FXobject root, FXobject endeff, FXobject goal, FXint align);

FX_API(FXint, fxGetItemEditMode)(void);
FX_API(FXint, fxGetItemListGroup)(void);

FX_API(FXint, fxKeyMove)(FXobject, FXint keyID, FXdouble frame, FXint flags);

#define _MESSIAH_PTR_MOTION	messiah_apicomponent = FX_APIFUNCS_MOTION;\
							FX_PTR(fxMotionNumKeys);\
							FX_PTR(fxMotionUpdate);\
							FX_PTR(fxChannelsGet);\
							FX_PTR(fxChannelsSet);\
							FX_PTR(fxChannelsBlend);\
							FX_PTR(fxChannelActiveGet);\
							FX_PTR(fxChannelActiveSet);\
							FX_PTR(fxChannelNumKeys);\
							FX_PTR(fxChannelSetup);\
							FX_PTR(fxChannelDataLink);\
							FX_PTR(fxChannelName);\
							FX_PTR(fxChannelRename);\
							FX_PTR(fxChannelOptions);\
							FX_PTR(fxChannelOptionsSet);\
							FX_PTR(fxChannelOptionsClear);\
							FX_PTR(fxKeyEditBegin);\
							FX_PTR(fxKeyEditEnd);\
							FX_PTR(fxKeyChannelsGet);\
							FX_PTR(fxKeyChannelsSet);\
							FX_PTR(fxKeyInclusionGet);\
							FX_PTR(fxKeyInclusionSet);\
							FX_PTR(fxKeySelected);\
							FX_PTR(fxKeySelect);\
							FX_PTR(fxKeyDeselect);\
							FX_PTR(fxKeyActiveGet);\
							FX_PTR(fxKeyActiveSet);\
							FX_PTR(fxKeyExists);\
							FX_PTR(fxKeyFrameGet);\
							FX_PTR(fxKeyTimeGet);\
							FX_PTR(fxKeyCreate);\
							FX_PTR(fxKeyChannelCreate);\
							FX_PTR(fxKeyDelete);\
							FX_PTR(fxChannelEndSet);\
							FX_PTR(fxChannelEndGet);\
							FX_PTR(fxKeyCurveSet);\
							FX_PTR(fxKeyCurveGet);\
							FX_PTR(fxKeyTangentsSet);\
							FX_PTR(fxKeyTangentsGet);\
							FX_PTR(fxMatrixGet);\
							FX_PTR(fxMatrixHPB);\
							FX_PTR(fxMatrixInit);\
							FX_PTR(fxMatrixTranslate);\
							FX_PTR(fxMatrixScale);\
							FX_PTR(fxMatrixRotateX);\
							FX_PTR(fxMatrixRotateY);\
							FX_PTR(fxMatrixRotateZ);\
							FX_PTR(fxKeyKneeConstraintPosV);\
							FX_PTR(fxKeyKneeConstraintPos);\
							FX_PTR(fxKeyKneeConstraint);\
							FX_PTR(fxKeyTargetPosV);\
							FX_PTR(fxKeyTargetPos);\
							FX_PTR(fxKeyTarget);\
							FX_PTR(fxKeyIK2DPosV);\
							FX_PTR(fxKeyIK2DPos);\
							FX_PTR(fxKeyIK2D);\
							FX_PTR(fxKeyTCBGet);\
							FX_PTR(fxKeyTCBSet);\
							FX_PTR(fxGetItemEditMode);\
							FX_PTR(fxGetItemListGroup);\
							FX_PTR(fxKeyMove);\
							FX_PTR(fxKeyChannelsSetBezierHandleOut);\
							FX_PTR(fxKeyChannelsSetBezierHandleIn);\
							FX_PTR(fxKeyChannelsGetBezierHandleOut);\
							FX_PTR(fxKeyChannelsGetBezierHandleIn);\
							FX_PTR(fxKeyFrameSetBezierHandleIn);\
							FX_PTR(fxKeyFrameSetBezierHandleOut);\
							FX_PTR(fxKeyFrameGetBezierHandleIn);\
							FX_PTR(fxKeyFrameGetBezierHandleOut)



#endif //__MESSIAH_MOT


#endif //_MESSIAH_API_MOTION
